
;==========================================================================
;
;                          QUICK
;
;       QRP  Under  Integrated  Control  from the  Keyer
;
;               by   Rod Kreuter   WA3ENK     9/16/98
;
;==========================================================================
;
	processor 16f84
		__config        _RC_OSC &_WDT_OFF &_PWRTE_ON
;
;==========================================================================
	LIST
; P16F84.INC  Standard Header File, Version 2.00    Microchip Technology, Inc.
	NOLIST

; This header file defines configurations, registers, and other useful bits of
; information for the PIC16F84 microcontroller.  These names are taken to match 
; the data sheets as closely as possible.  

; Note that the processor must be selected before this file is 
; included.  The processor may be selected the following ways:

;       1. Command line switch:
;               C:\ MPASM MYFILE.ASM /PIC16F84
;       2. LIST directive in the source file
;               LIST   P=PIC16F84
;       3. Processor Type entry in the MPASM full-screen interface

;==========================================================================
;
;       Revision History
;
;==========================================================================

;Rev:   Date:    Reason:

;2.00   07/24/96 Renamed to reflect the name change to PIC16F84.
;1.01   05/17/96 Corrected BADRAM map
;1.00   10/31/95 Initial Release

;==========================================================================
;
;       Register Definitions
;
;==========================================================================

W                            EQU     H'0000'
F                            EQU     H'0001'

;----- Register Files------------------------------------------------------

INDF                         EQU     H'0000'
TMR0                         EQU     H'0001'
PCL                          EQU     H'0002'
STATUS                       EQU     H'0003'
FSR                          EQU     H'0004'
PORTA                        EQU     H'0005'
PORTB                        EQU     H'0006'
EEDATA                       EQU     H'0008'
EEADR                        EQU     H'0009'
PCLATH                       EQU     H'000A'
INTCON                       EQU     H'000B'

OPTION_REG                   EQU     H'0081'
TRISA                        EQU     H'0085'
TRISB                        EQU     H'0086'
EECON1                       EQU     H'0088'
EECON2                       EQU     H'0089'

;----- STATUS Bits --------------------------------------------------------

IRP                          EQU     H'0007'
RP1                          EQU     H'0006'
RP0                          EQU     H'0005'
NOT_TO                       EQU     H'0004'
NOT_PD                       EQU     H'0003'
Z                            EQU     H'0002'
DC                           EQU     H'0001'
C                            EQU     H'0000'

;----- INTCON Bits --------------------------------------------------------

GIE                          EQU     H'0007'
EEIE                         EQU     H'0006'
T0IE                         EQU     H'0005'
INTE                         EQU     H'0004'
RBIE                         EQU     H'0003'
T0IF                         EQU     H'0002'
INTF                         EQU     H'0001'
RBIF                         EQU     H'0000'

;----- OPTION Bits --------------------------------------------------------

NOT_RBPU                     EQU     H'0007'
INTEDG                       EQU     H'0006'
T0CS                         EQU     H'0005'
T0SE                         EQU     H'0004'
PSA                          EQU     H'0003'
PS2                          EQU     H'0002'
PS1                          EQU     H'0001'
PS0                          EQU     H'0000'

;----- EECON1 Bits --------------------------------------------------------

EEIF                         EQU     H'0004'
WRERR                        EQU     H'0003'
WREN                         EQU     H'0002'
WR                           EQU     H'0001'
RD                           EQU     H'0000'

;==========================================================================
;
;       RAM Definition
;
;==========================================================================

	__MAXRAM H'CF'
	__BADRAM H'07', H'50'-H'7F', H'87'

;==========================================================================
;
;       Configuration Bits
;
;==========================================================================

_CP_ON                       EQU     H'000F'
_CP_OFF                      EQU     H'3FFF'
_PWRTE_ON                    EQU     H'3FF7'
_PWRTE_OFF                   EQU     H'3FFF'
_WDT_ON                      EQU     H'3FFF'
_WDT_OFF                     EQU     H'3FFB'
_LP_OSC                      EQU     H'3FFC'
_XT_OSC                      EQU     H'3FFD'
_HS_OSC                      EQU     H'3FFE'
_RC_OSC                      EQU     H'3FFF'
  
		LIST

  ; PortA

Dit_Paddle              EQU     H'0'
Dah_Paddle              EQU     H'1'
Mode_Switch             EQU     H'2'
Play_Rec                EQU     H'3'

  ; PortB

Key_Out                 EQU     H'0'    ; drives the transmitter 
TR_Relay                EQU     H'1'    ; transmitt/recieve relay
RIT_Out                 EQU     H'2'    ; shifts the VFO from T/R
Mute                    EQU     H'3'    ; used during T/R switching
Audio                   EQU     H'4'    ; used to drive the sidetone Piezo
Out_Led                 EQU     H'5'    ; to shown when you're in set up


Flags                   EQU     H'1E'
Dit_Flag                EQU     H'0'
Dah_Flag                EQU     H'1'
SetUp_Flag              EQU     H'2'
First_Flag              EQU     H'3'    ; used in record to wait for input
Mute_Audio              EQU     H'4'    ; used to disable sidetone

J                       EQU     H'1F'   ; general purpose counter
Freq                    EQU     H'20'   ; sidetone pitch
Dit_Length              EQU     H'21'   
Dah_Length              EQU     H'22'      
Delta                   EQU     H'23'   ; used in calculating dah length
Mess_Addr               EQU     H'24'   ; address for EEPROM access
Mess_Data               EQU     H'25'
EE_Shift                EQU     H'26'   ; used to say when to store
Num_Bytes               EQU     H'27'   ; length of recorded message
WTemp                   EQU     H'28'   ; save working reg

;==========================================================================
		org     0

		clrf    PORTB
		clrf    PORTA
		bsf     STATUS, RP0     ;goto bank 1
		movlw   H'00'
		movwf   TRISB           ;make PortB output
		movlw   B'00011111'
		movwf   TRISA           ;make PortA input
		bcf     STATUS, RP0     ;go back to bank 0
;==========================================================================
; set up the timer       
; the timer is used to time the dit and dah length

		clrwdt
		bsf     STATUS, RP0
		movlw   B'10000111'     ;set timer and prescaler
		movwf   OPTION_REG
		bcf     STATUS, RP0     ;go back to bank 0
;==========================================================================
; preset some constants

		movlw   D'25'           ; set sidetone pitch
		movwf   Freq
		movlw   D'210'          ; 255 - 45 = 210
		movwf   Dit_Length
		movlw   D'120'          ; 255 - 3*(45) = 120
		movwf   Dah_Length
		clrf    Flags           ; reset all flags
;==========================================================================
		goto   No_WA3ENK
; send my call
		call    Send_Dit        ; W
		call    Send_Dah
		call    Send_Dah
		call    Wait3
		

		call    Send_Dit        ; A
		call    Send_Dah
		call    Wait3
		

		call    Send_Dit        ; 3
		call    Send_Dit
		call    Send_Dit
		call    Send_Dah
		call    Send_Dah
		call    Wait3
		
		call    Send_Dit        ; E
		call    Wait3
		

		call    Send_Dah        ; N
		call    Send_Dit
		call    Wait3
		

		call    Send_Dah        ; K
		call    Send_Dit
		call    Send_Dah
		call    Wait3
		
No_WA3ENK
;==========================================================================

Main_Loop       
		btfss   PORTA, Play_Rec
		call    Message

		btfss   PORTA, Mode_Switch
		call    Set_Up

		btfss   PORTA, Dah_Paddle
		call    Send_Dah

		btfsc   Flags, Dah_Flag
		call    Send_Dah

		btfss   PORTA, Dit_Paddle
		call    Send_Dit

		btfsc   Flags, Dit_Flag
		call    Send_Dit

		goto    Main_Loop

;==========================================================================

Send_Dit    
		bsf     PORTB,  Mute       ; mute audio for QSK
		call    Wait1Ms            ; waste a little time
		bsf     PORTB,  RIT_Out    ; shift the VFO
		call    Wait1Ms            ; waste a little time
		bsf     PORTB,  TR_Relay   ; switch the antenna             
		call    Wait1Ms            ; waste a little time

		clrf    INTCON
		movf    Dit_Length, W      ; since the counter counts
		movwf   TMR0               ; up instead of down, the number 
								   ; is 255 - number

		btfss   Flags,  SetUp_Flag ; in set-up don't key the xmitter    
		bsf     PORTB,  Key_Out    ; turn on the transmitter

		bcf     Flags, Dit_Flag    ; clear the dit flag 

Make_Dit_Tone   
		btfss   Flags,  Mute_Audio  ; sidetone enabled?
		bsf     PORTB, Audio        ; set the audio bit
		movf    Freq, W             ; put tone var in W
		movwf   J
J1Loop  
		decfsz  J, f               ; j=j-1 
		goto    J1Loop

		btfss   PORTA, Dah_Paddle  ; look at dah paddle
		bsf     Flags, Dah_Flag    ; make Dah_Flag=1 if
								   ; the paddle is closed

		btfsc   INTCON,T0IF        ; has timer rolled?
		goto    Exit_Dit

		bcf     PORTB, Audio       ; clear audio bit
		movf    Freq, W 
		movwf   J
J2Loop  
		decfsz  J, f               ;j=j-1 
		goto    J2Loop

		btfss   PORTA, Dah_Paddle  ; look at dah paddle
		bsf     Flags, Dah_Flag    ; make Dah_Flag=1 if
								   ; the paddle is closed
	
		btfss   INTCON, T0IF       ; if T0IF is set, it overflowed
		goto    Make_Dit_Tone

Exit_Dit        
		clrf    INTCON
		movf    Dit_Length, W      ; set delay time ff - delay
		movwf   TMR0               ; start the timer for the space

		bcf     PORTB, Key_Out     ; turn the transmitter off
		call    Wait10Ms           ; wait for fall time of xmitter  

		btfss   PORTA, Dah_Paddle  ; look at dah paddle
		bsf     Flags, Dah_Flag    ; make Dah_Flag=1 if
								   ; the paddle is closed

		bcf     PORTB,  TR_Relay   ; switch the antenna
		call    Wait1Ms            ; waste a little time
		bcf     PORTB,  RIT_Out    ; shift the VFO
		call    Wait1Ms            ; waste a little time
		bcf     PORTB,  Mute       ; un-mute the audio              
					
Make_Dit_Space 
		btfss   PORTA, Dah_Paddle  ; look at dah paddle
		bsf     Flags, Dah_Flag    ; make Dah_Flag=1 if
								   ; the paddle is closed
		btfsc   INTCON,T0IF        ; has timer rolled?
		goto    Exit_Dit_Space

		goto    Make_Dit_Space

Exit_Dit_Space
		return
;==========================================================================

Send_Dah    
		bsf     PORTB,  Mute       ; mute audio for QSK
		call    Wait1Ms            ; waste a little time
		bsf     PORTB,  RIT_Out    ; shift the VFO
		call    Wait1Ms            ; waste a little time
		bsf     PORTB,  TR_Relay   ; switch the antenna             
		call    Wait1Ms            ; waste a little time

		clrf    INTCON
		movf    Dah_Length, W     ; set dah delay      ;ff - delay
		movwf   TMR0    


		btfss   Flags, SetUp_Flag ; if setup don't key xmit
		bsf     PORTB, Key_Out    ; turn on the transmitter

		bcf     Flags, Dah_Flag   ; clear dah flag

Make_Dah_Tone   
		btfss   Flags,  Mute_Audio  ; sidetone enabled?
		bsf     PORTB, Audio        ; set the audio bit
		movf    Freq, W             ; put tone var in W
		movwf   J
J5Loop  
		decfsz  J, F               ; j=j-1 
		goto    J5Loop

		btfss   PORTA, Dit_Paddle  ; look at dit paddle
		bsf     Flags, Dit_Flag    ; make Dit_Flag=1 if
								   ; the paddle is closed
		btfsc   INTCON,T0IF        ; has timer rolled?
		goto    Exit_Dah

		bcf     PORTB, Audio       ; clear audio bit
		movf    Freq, W 
		movwf   J
J6Loop  
		decfsz  J, F               ; j=j-1 
		goto    J6Loop 

		btfss   PORTA, Dit_Paddle  ; look at dit paddle
		bsf     Flags, Dit_Flag    ; make Dit_Flag=1 if
								   ; the paddle is closed

		btfss   INTCON, T0IF       ; if T0IF is set, it overflowed
		goto    Make_Dah_Tone

Exit_Dah        
		clrf    INTCON
		movf    Dit_Length, W      ; even dah has a dit space
		movwf   TMR0    

		bcf     PORTB, Key_Out     ; turn the transmitter off
		call    Wait10Ms           ; wait for xmitter fall time

		btfss   PORTA, Dit_Paddle  ; look at dit paddle
		bsf     Flags, Dit_Flag    ; make Dit_Flag=1 if
								   ; the paddle is closed
		bcf     PORTB,  TR_Relay   ; switch the antenna
		call    Wait1Ms            ; waste a little time
		bcf     PORTB,  RIT_Out    ; shift the VFO
		call    Wait1Ms            ; waste a little time
		bcf     PORTB,  Mute       ; un-mute the audio              
			
Make_Dah_Space  
		btfss   PORTA, Dit_Paddle  ; look at dit paddle
		bsf     Flags, Dit_Flag    ; make Dit_Flag=1 if
								   ; the paddle is closed

		btfsc   INTCON,T0IF        ; if T0IF is set, it overflowed
		goto    Exit_Dah_Space
		
		goto    Make_Dah_Space

Exit_Dah_Space
			return
;==========================================================================
; This first routine is used to change the keyer speed

Set_Up      
		bsf     PORTB, Out_Led     ; turn the Led on
		bsf     Flags, SetUp_Flag

		call    Send_Dit          ; S
		call    Send_Dit
		call    Send_Dit
		call    Wait3
	
		
SS_Loop 
		btfss   PORTA, Mode_Switch
		goto    Set_Freq              ; we're done, lets go on        
		
		btfss   PORTA, Dah_Paddle
		call    Dec_Speed

		btfss   PORTA, Dit_Paddle
		call    Inc_Speed
		
		goto    SS_Loop

;===========================================

Dec_Speed       
		movlw   D'175'         ; 175 is about 8 WPM (500 KHZ Clk)
		subwf   Dit_Length, W
		btfsc   STATUS, C
		decf    Dit_Length, F
		call    Send_Dit

		movf    Dit_Length, W
		sublw   D'255'
		movwf   Delta
		addwf   Delta, W    ; w = 2 * Delta
		addwf   Delta, W    ; w=  3 * Delta
		sublw   D'255'
		movwf   Dah_Length
				
		return

;===========================================

Inc_Speed       
		movlw   D'235'      ; 235 is about 32 WPM (500 KHZ Clk)     
		subwf   Dit_Length, W
		btfss   STATUS, C
		incf    Dit_Length, F
		call    Send_Dit        

		movf    Dit_Length, W
		sublw   D'255'
		movwf   Delta
		addwf   Delta, W    ; w = 2 * Delta
		addwf   Delta, W    ; w=  3 * Delta
		sublw   D'255'
		movwf   Dah_Length

		return

;==========================================================================
; This routine changes the pitch of the sidetone

Set_Freq    
		call    Send_Dit       ; F
		call    Send_Dit
		call    Send_Dah
		call    Send_Dit
		call    Wait3
	


FS_Loop     
		btfss   PORTA, Mode_Switch
		goto    Tune             ; we're done, lets go on       
				
		btfss   PORTA, Dah_Paddle
		call    Inc_Freq

		btfss   PORTA, Dit_Paddle
		call    Dec_Freq
		
		goto    FS_Loop

;===========================================
		
Dec_Freq        
		movlw   D'9'        ; don't let freq be less than 8
		subwf   Freq, W     ; result in W
		btfsc   STATUS, C
		decf    Freq, F
		call    Send_Dit
		return
;===========================================

Inc_Freq        
		movlw   D'50'
		subwf   Freq, W
		btfss   STATUS, C
		incf    Freq, F
		call    Send_Dit
		return
;==========================================================================
; This routine turns the transmitter on (key down). It's used to
; allow some time to tune an antenna tuner, for example.

Tune        
		call    Send_Dah       ; T
		call    Wait3
	
Tune_Loop   
		
		btfss   PORTA, Mode_Switch
		goto    Exit_Tune        ; we're done, lets go back        
				
		btfss   PORTA, Dah_Paddle
		call    Xmit_Off

		btfss   PORTA, Dit_Paddle
		call    Xmit_On
		
		goto    Tune_Loop

Exit_Tune   
		goto    SideTone_Mute

;======================================
Xmit_On     
		bsf     PORTB,  Mute       ; mute audio for QSK
		call    Wait1Ms            ; waste a little time
		bsf     PORTB,  RIT_Out    ; shift the VFO
		call    Wait1Ms            ; waste a little time
		bsf     PORTB,  TR_Relay   ; switch the antenna             
		call    Wait1Ms            ; waste a little time
		bsf     PORTB, Key_Out     ; turn on the transmitter

Make_Tune_Tone  
		bsf     PORTB, Audio       ; set the audio bit
		movf    Freq, W            ; put tone var in W
		movwf   J
T1Loop          
		decfsz  J, f               ; j=j-1 
		goto    T1Loop

		bcf     PORTB, Audio       ; clear audio bit
		movf    Freq, W 
		movwf   J
T2Loop          
		decfsz  J, f               ;j=j-1 
		goto    T2Loop
		
		btfsc   PORTA, Dah_Paddle
		goto    Make_Tune_Tone

		
Xmit_Off    
		bcf     PORTB, Key_Out     ; turn off the transmitter
		call    Wait10Ms           ; let it settle
		bcf     PORTB,  TR_Relay   ; switch the antenna             
		call    Wait1Ms            ; waste a little time
		bcf     PORTB,  RIT_Out    ; shift the VFO
		call    Wait1Ms            ; waste a little time
		bcf     PORTB,  Mute       ; mute audio for QSK
		call    Wait1Ms            ; waste a little time

		return
;==========================================================================
; This routine mutes the audio (sidetone)
; the LED blinks during this routine in case you turn off the sidetone
; and can't get it back on again

SideTone_Mute   
		call    Send_Dit        ; A
		call    Send_Dah
		call    Wait3
	
Mute_Loop   
		bcf     PORTB,  Out_Led     ; blink the led
		call    Wait10Ms
		call    Wait10Ms
		call    Wait10Ms
		call    Wait10Ms
		bsf     PORTB,  Out_Led
		call    Wait10Ms
		call    Wait10Ms
		call    Wait10Ms
		call    Wait10Ms

		btfss   PORTA, Mode_Switch
		goto    Exit_Mute               ; we're done, lets go         
				
		btfss   PORTA, Dah_Paddle       ; enable sidetone
		bcf     Flags, Mute_Audio

		btfss   PORTA, Dit_Paddle       ; disable sidetone
		bsf     Flags, Mute_Audio
	
		goto    Mute_Loop
		
Exit_Mute   
		bcf     PORTB, Out_Led  ; don't forget to turn out the light
		call    Send_Dah        ; K  got back to keyer
		call    Send_Dit
		call    Send_Dah
		call    Wait3
	

		bcf     Flags,  SetUp_Flag    ; can't clear them all
		bcf     Flags,  Dit_Flag      ; because of the mute
		bcf     Flags,  Dah_Flag      ; flag
		return
	
;==========================================================================
; The elements are stored as follows:
;      End  =   00
;      Dit =    01
;      Space =  10
;      Dah =    11
; This is the routine which "records" and "plays back" short messages which
; are stored in non-volatile EEROM. Four "elements" are stored in a byte.
	
Message
		bsf     PORTB, Out_Led          ; turn the Led on
		bsf     Flags, SetUp_Flag
		clrf    Mess_Addr
		incf    Mess_Addr               ; skip the first one (store length)
		clrf    EE_Shift                ; set shift = 0
		bsf     Flags,  First_Flag

		call    Send_Dit        ; R record
		call    Send_Dah
		call    Send_Dit
		call    Wait3
	

Rec_Start   
		clrf    INTCON              ; set up the timer for spaces
		movf    Dah_Length, W       ; bigger space
		movwf   TMR0    

Rec_Loop    
		btfsc   EE_Shift, 2      ; look to see if it's time to write
		call    Write
		
		btfss   PORTA, Play_Rec  ; Play_Rec switch pushed?
		goto    Exit_Rec         ; yes, we're done, lets go back        

		btfsc   Mess_Addr, 6    ; is the length up to 64 bytes?
		goto    Exit_Rec        ; memory's full (only have 64 bytes)
		
		btfsc   PORTA, Dah_Paddle
		goto    No_Dah
		call    Send_Dah
		incf    EE_Shift, F
		bsf     STATUS, C        ; set carry bit
		rrf     Mess_Data, F
		bsf     STATUS, C        ; set carry bit
		rrf     Mess_Data, F
		bcf     Flags,  First_Flag
		goto    Rec_Start

No_Dah      
		btfsc   PORTA, Dit_Paddle
		goto    No_Dit
		call    Send_Dit
		incf    EE_Shift, F
		bcf     PORTB,  Out_Led      ; turn it off
		bsf     STATUS, C            ; set carry
		rrf     Mess_Data, F
		bcf     STATUS, C            ; clear carry
		rrf     Mess_Data, F        
		bcf     Flags,  First_Flag
		goto    Rec_Start

No_Dit      
		btfsc   Flags,  First_Flag
		goto    Rec_Start
		btfss   INTCON,T0IF
		goto    Rec_Loop              ; go back if time is not up
		incf    EE_Shift, F
		bcf     PORTB,  Out_Led       ; turn it off
		bcf     STATUS, C             ; clear carry
		rrf     Mess_Data, F
		bsf     STATUS, C             ; set carry
		rrf     Mess_Data, F        
		
		bsf     PORTB, Out_Led        ; turn the Led on 
		call    Wait10Ms
		bcf     PORTB, Out_Led        ; turn the Led off  
		goto    Rec_Start

Exit_Rec    
		btfsc   Flags,  First_Flag
		goto    No_Rec
		
More_End    
		bcf     STATUS, C             ; clear carry bit
		rrf     Mess_Data, F
		bcf     STATUS, C             ; clear carry bit
		rrf     Mess_Data, F

End_Loop    
		incf    EE_Shift, F
		btfss   EE_Shift, 2     ; look to see if it's time to write
		goto    More_End        ; stuff the last byte with 00
		call    Write           ; write end character
		movf    Mess_Addr, W    ; put the last addr in W
		movwf   Mess_Data       ; put the last address in Mess_Data
		clrf    Mess_Addr
		call    Write           ; write address in first byte

No_Rec      
		bsf     PORTB, Out_Led     ; turn the Led on 
		
;================================================
; This routine plays the stored message the LED also blinks once       
; to let you know your here

		call    Send_Dit          ; P Play
		call    Send_Dah
		call    Send_Dah
		call    Send_Dit
		bcf     PORTB, Out_Led        ; turn the Led off
		call    Wait3                 ; blink LED in case audio     
		call    Wait3                 ; has been disabled   
		bsf     PORTB, Out_Led        ; turn the Led on
		
		bcf     Flags, SetUp_Flag


Play_Loop   
		btfss   PORTA, Play_Rec     ; button pushed?   
		goto    Exit_Play           ; we're done, lets go back        
				
		btfss   PORTA, Dah_Paddle
		goto    Exit_Play

		btfss   PORTA, Dit_Paddle
		goto    Play_Message
		
		goto    Play_Loop


Exit_Play   
		bcf     PORTB, Out_Led      ; don't forget to turn out the light    
		call    Send_Dah            ; K  go back to keyer
		call    Send_Dit
		call    Send_Dah
		call    Wait3

		bcf     Flags,  SetUp_Flag    ; can't clear them all
		bcf     Flags,  Dit_Flag      ; because of the mute
		bcf     Flags,  Dah_Flag      ; flag

		return

;==========================================

Play_Message    
		clrf    Mess_Addr
		call    Read            ; get the number of bytes
		movwf   Num_Bytes       ; put length in Num_Bytes

		clrf    EE_Shift
		call    Read
				
More_Play   
		btfsc   Mess_Addr, 6      ; 64 bytes??
		goto    Play_Loop
		
		btfsc   PORTA, Play_Rec   ; button pushed?  
		goto    No_Abort
		call    Wait3             ; need to de-bounce the switch    
		goto    Play_Loop         ; abort playback
	
No_Abort    
		decfsz  Num_Bytes, F
		goto    Continue1
		goto    Play_Loop         ; we're done so go back

			
Continue1   
		btfsc   EE_Shift, 2
		call    Read
		
		rrf     Mess_Data, F      ; first shift
		btfss   STATUS, C
		goto    LP1
		goto    LP2

LP1     
		rrf     Mess_Data, F      ; second shift first was a 0
		btfss   STATUS, C
		goto    LP3
		goto    LP4

LP2     
		rrf     Mess_Data, F      ; second shift  first was a 1
		btfss   STATUS, C
		goto    LP5
		goto    LP6

LP3     
		incf    EE_Shift, F
		goto    Play_Loop         ; end character 00
		
LP4     
		call    Wait              ; space character 10
		call    Wait
		call    Wait
		incf    EE_Shift, F
		goto    More_Play

LP5     
		call    Send_Dit          ; dit 01
		incf    EE_Shift, F
		goto    More_Play

LP6     
		call    Send_Dah          ; dah 11
		incf    EE_Shift, F     
		goto    More_Play   

			
;==========================================================================
; EEPROM Read and Write Routines

Read        
		movf    Mess_Addr, W
		movwf   EEADR
		bsf     STATUS, RP0
		bsf     EECON1, RD
		bcf     STATUS, RP0
		movf    EEDATA, W
		incf    Mess_Addr, F      ; inc and put back
		clrf    EE_Shift
		movwf   Mess_Data
		return

Write       
		movf    Mess_Addr, W
		movwf   EEADR
		movf    Mess_Data, W
		movwf   EEDATA
		bsf     STATUS, RP0
		bsf     EECON1, WREN
		movlw   H'55'
		movwf   EECON2
		movlw   H'AA'
		movwf   EECON2
		bsf     EECON1, WR
		bcf     EECON1, WREN
		bcf     STATUS, RP0

		incf    Mess_Addr, F      ; inc and put back
		clrf    EE_Shift
		return

;==========================================================================
Wait1Ms         
		movwf   WTemp           ; save W
		movlw   D'42'           ; based on 500 KHz clock
		movwf   J
W1Ms_Loop       
		decfsz  J, F            ; j=j-1 
		goto    W1Ms_Loop   
		movf    WTemp, W        ; restore W
		return
;==========================================================================
		
Wait10Ms        
		movwf   WTemp           ; save W
		movlw   D'250'          ; based on 500 KHz clock
		movwf   J
W10Ms_Loop  
		nop
		nop
		decfsz  J, F            ; j=j-1 
		goto    W10Ms_Loop  
		movf    WTemp, W        ; restore W
		return
;==========================================================================

Wait3       
		call    Wait
		call    Wait
		call    Wait
		return
;==========================================================================
Wait            
		clrf    INTCON
		movf    Dit_Length, W      
		movwf   TMR0    
		
Wait_A_Bit  
		btfss   INTCON,T0IF
		goto    Wait_A_Bit       ; loop until the timer times out  

		return
		end
	
